@@ -37,14 +37,20 @@ require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(new_user_amazon run_aws_cmd get_aws_security_id
                  launch_server $configure_aws1 $pem_file $credpath
-                 $aws_configure $aws);
+                 $aws_configure $aws connect_shell);
 our $pem_file='';
 our $credpath='.';
 our $aws={};
 
 sub cmd_raw {
 
-   return Net::FullAuto::FA_Core::cmd_raw(@_);
+   return &Net::FullAuto::FA_Core::cmd_raw(@_);
+
+}
+
+sub connect_shell {
+
+   return &Net::FullAuto::FA_Core::connect_shell(@_);
 
 }
 
