@@ -59,6 +59,7 @@ my $configure_gnusocial=sub {
       $permanent_ip=$hash->{PublicIp}; 
    }
    my $handle=$main::aws->{$server_type}->[$cnt]->[1];
+   my $local=connect_shell();
    my ($stdout,$stderr)=('','');
    ($stdout,$stderr)=$handle->cmd("sudo yum -y update",'__display__');
    ($stdout,$stderr)=$handle->cmd("sudo yum clean all",'__display__');
@@ -66,7 +67,7 @@ my $configure_gnusocial=sub {
    ($stdout,$stderr)=$handle->cmd("sudo yum groups mark convert",
       '__display__');
    ($stdout,$stderr)=$handle->cmd(
-      'sudo yum -y install cyrus-sasl-plain send-mail-cf m4 java java-devel',
+      'sudo yum -y install cyrus-sasl-plain sendmail-cf m4 java java-devel',
       '__display__');
    my $install_gnusocial=<<'END';
 
@@ -220,40 +221,44 @@ print "\n\n\n\n\n\n\nWE SHOULD HAVE INSTALLED MARIADB=$stdout<==\n\n\n\n\n\n\n";
          "programming/pcre/$pcre.tar.gz",'__display__');
       ($stdout,$stderr)=$handle->cmd("sudo tar xvf $pcre.tar.gz",'__display__');
       last unless $stderr;
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rfv $pcre.tar.gz",'__display__');
    }
    my $zlib='zlib-1.2.8';
-   ($stdout,$stderr)=$handle->cmd("sudo wget --random-wait --progress=dot ".
-      "http://zlib.net/$zlib.tar.gz",'__display__');
-   $checksum='44d667c142d7cda120332623eab69f40';
-   ($stdout,$stderr)=$handle->cmd(
-      "sudo md5sum -c - <<<\"$checksum $zlib.tar.gz\"",
-      '__display__');
-   unless ($stderr) {
-      print(qq{ + CHECKSUM Test for $zlib *PASSED* \n});
-   } else {
+   foreach my $count (1..3) {
+      ($stdout,$stderr)=$handle->cmd("sudo wget --random-wait --progress=dot ".
+         "http://zlib.net/$zlib.tar.gz",'__display__');
+      $checksum='44d667c142d7cda120332623eab69f40';
+      ($stdout,$stderr)=$handle->cmd(
+         "sudo md5sum -c - <<<\"$checksum $zlib.tar.gz\"",
+         '__display__');
+      unless ($stderr) {
+         print(qq{ + CHECKSUM Test for $zlib *PASSED* \n});
+         last
+      } elsif ($count>=3) {
+         print "FATAL ERROR! : CHECKSUM Test for $zlib.tar.gz *FAILED* ",
+               "after $count attempts\n";
+         &Net::FullAuto::FA_Core::cleanup;
+      }
       ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $zlib.tar.gz",'__display__');
-      my $dc=1;
-      print "FATAL ERROR! : CHECKSUM Test for $zlib.tar.gz *FAILED* ",
-            "after $dc attempts\n";
-      &Net::FullAuto::FA_Core::cleanup;
    }
    ($stdout,$stderr)=$handle->cmd("sudo tar xvf $zlib.tar.gz",'__display__');
    my $ossl='openssl-1.0.2h';
-   $checksum='577585f5f5d299c44dd3c993d3c0ac7a219e4949';
-   ($stdout,$stderr)=$handle->cmd("sudo wget --random-wait --progress=dot ".
-      "https://www.openssl.org/source/$ossl.tar.gz",
-      '__display__');
-   ($stdout,$stderr)=$handle->cmd(
-      "sudo sha1sum -c - <<<\"$checksum $ossl.tar.gz\"",
-      '__display__');
-   unless ($stderr) {
-      print(qq{ + CHECKSUM Test for $ossl *PASSED* \n});
-   } else {
+   foreach my $count (1..3) {
+      $checksum='577585f5f5d299c44dd3c993d3c0ac7a219e4949';
+      ($stdout,$stderr)=$handle->cmd("sudo wget --random-wait --progress=dot ".
+         "https://www.openssl.org/source/$ossl.tar.gz",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd(
+         "sudo sha1sum -c - <<<\"$checksum $ossl.tar.gz\"",'__display__');
+      unless ($stderr) {
+         print(qq{ + CHECKSUM Test for $ossl *PASSED* \n});
+         last
+      } elsif ($count>=3) {
+         print "FATAL ERROR! : CHECKSUM Test for $ossl.tar.gz *FAILED* ",
+               "after $count attempts\n";
+         &Net::FullAuto::FA_Core::cleanup;
+      }
       ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $ossl.tar.gz",'__display__');
-      my $dc=1;
-      print "FATAL ERROR! : CHECKSUM Test for $ossl.tar.gz *FAILED* ",
-            "after $dc attempts\n";
-      &Net::FullAuto::FA_Core::cleanup;
    }
    ($stdout,$stderr)=$handle->cmd("sudo tar xvf $ossl.tar.gz",'__display__');
    ($stdout,$stderr)=$handle->cwd("../..");
@@ -464,18 +469,6 @@ END
    ($stdout,$stderr)=$handle->cmd(
        "${sudo}sed -i \'s/^        listen       80/        listen       ".
        "\*:443 ssl default_server/\' /usr/local/nginx/nginx.conf");
-   #($stdout,$stderr)=$handle->cmd("${sudo}sed -i 's/SCRIPT_NAME/PATH_INFO/' ".
-   #    '/usr/local/nginx/nginx.conf');
-   #$ad='# Catalyst requires setting PATH_INFO (instead of SCRIPT_NAME)'.
-   #    ' to \$fastcgi_script_name';
-   #($stdout,$stderr)=$handle->cmd(
-   #    "${sudo}sed -i \'/PATH_INFO/i$ad\' /usr/local/nginx/nginx.conf");
-   #$ad='fastcgi_param  SCRIPT_NAME        /;';
-   #($stdout,$stderr)=$handle->cmd(
-   #    "${sudo}sed -i \'/PATH_INFO/a$ad\' /usr/local/nginx/nginx.conf");
-   #($stdout,$stderr)=$handle->cmd(
-   #    "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
-   #    '/usr/local/nginx/nginx.conf');
    ($stdout,$stderr)=$handle->cmd(
        "${sudo}sed -i \'s/nobody/ec2-user/\' ".
        '/usr/local/nginx/nginx.conf');
@@ -628,6 +621,8 @@ print "\n\n\n\n\n\n\nSTDERR=$stderr\n\n\n\n\n\n\n";
    $c="aws iam create-access-key --user-name gnusocial_email";
    ($hash,$output,$error)=run_aws_cmd($c);
    $hash||={};
+   my $access_id=$hash->{AccessKey}->{AccessKeyId};
+   my $secret_access_key=$hash->{AccessKey}->{SecretAccessKey};
    my $java_smtp_generator=<<END;
 import javax.crypto.Mac;
 import javax.crypto.spec.SecretKeySpec;
@@ -682,9 +677,42 @@ END
       "echo -e \"$java_smtp_generator\" > SesSmtpCredentialGenerator.java");
    ($stdout,$stderr)=$handle->cmd("javac SesSmtpCredentialGenerator.java");
    $handle->cmd_raw(
-      "export AWS_SECRET_ACCESS_KEY=$hash->{AccessKey}->{SecretAccessKey}");
+      "export AWS_SECRET_ACCESS_KEY=$secret_access_key");
    my $smtppass='';
    ($smtppass,$stderr)=$handle->cmd("java SesSmtpCredentialGenerator");
+   my $sespolicy=<<END;
+{
+   \\x22Version\\x22:\\x222012-10-17\\x22,
+   \\x22Statement\\x22: [{
+        \\x22Effect\\x22:\\x22Allow\\x22,
+        \\x22Action\\x22:\\x22ses:SendRawEmail\\x22,
+        \\x22Resource\\x22:\\x22*\\x22
+}]}
+END
+   chop $sespolicy;
+   ($stdout,$stderr)=$local->cmd(
+      "echo -e \"$sespolicy\" > ./sespolicy");
+   $c="aws iam list-policies";
+   ($hash,$output,$error)=run_aws_cmd($c);
+   $hash||={};
+   foreach my $policy (@{$hash->{Policies}}) {
+      if ($policy->{PolicyName} eq 'sespolicy') {
+         $c="aws iam detach-user-policy --user-name gnusocial_email ".
+            "--policy-arn $policy->{Arn}";
+         ($hash,$output,$error)=run_aws_cmd($c);
+         $c="aws iam delete-policy --policy-arn $policy->{Arn}";
+         ($hash,$output,$error)=run_aws_cmd($c);
+         last;
+      }
+   }
+   $c="aws iam create-policy --policy-name sespolicy --policy-document ".
+      "file://sespolicy";
+   ($hash,$output,$error)=run_aws_cmd($c);
+   my $policy_arn=$hash->{Policy}->{Arn};
+   $c="aws iam attach-user-policy --user-name gnusocial_email ".
+      "--policy-arn $policy_arn";
+   ($hash,$output,$error)=run_aws_cmd($c);
+   ($stdout,$stderr)=$local->cmd("rm -rfv ./sespolicy",'__display__'); 
    use LWP::UserAgent;
    use HTTP::Request::Common;
    use IO::Socket::SSL qw();
@@ -757,14 +785,81 @@ END
          }
       }
    }
-print "\nSMTP SERVER = $smtp_server and\nVERIFIED_EMAIL = $verified_email and\nSMTPPASS=$smtppass and\nID=$hash->{AccessKey}->{AccessKeyId}\n";
-   print "\n   ACCESS GNU SOCIAL UI AT:\n\n",
-         " https://$public_ip/index.php\n";
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}touch /etc/mail/authinfo");
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}chmod 666 /etc/mail/authinfo");
    my $authinfo=<<END;
-AuthInfo:\\x24smtp_server \\x22U:root\\x22 \\x22I:\\x24hash->{AccessKey}->{AccessKeyId}\\x22 \\x22P:\\x24smtppass\\x22 \\x22M:PLAIN\\x22
+AuthInfo:$smtp_server \\x22U:root\\x22 \\x22I:$access_id\\x22 \\x22P:$smtppass\\x22 \\x22M:PLAIN\\x22
+END
+   chop $authinfo;   
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}echo -e \"$authinfo\" > /etc/mail/authinfo");
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}makemap -v hash /etc/mail/authinfo.db < /etc/mail/authinfo",
+      '__display__');
+   my $access="Connect:$smtp_server RELAY";
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}chmod -v 666 /etc/mail/access");
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}echo -e \"$access\" >> /etc/mail/access");
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}chmod -v 644 /etc/mail/access");
+   $ad="define(`SMART_HOST%SQ%, `$smtp_server%SQ%)dnl%NL%".
+       "define(`RELAY_MAILER_ARGS%SQ%, `TCP \$h 25%SQ%)dnl%NL%".
+       "define(`confAUTH_MECHANISMS%SQ%, `LOGIN PLAIN%SQ%)dnl%NL%".
+       "FEATURE(`authinfo%SQ%, `hash -o /etc/mail/authinfo.db%SQ%)dnl%NL%".
+       "MASQUERADE_AS(`fullauto.com%SQ%)dnl%NL%".
+       "FEATURE(masquerade_envelope)dnl%NL%".
+       "FEATURE(masquerade_entire_domain)dnl";
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i ".
+      "\'/MAILER(smtp)dnl/i$ad\' /etc/mail/sendmail.mc");
+   ($stdout,$stderr)=$handle->cmd(
+       "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
+       '/etc/mail/sendmail.mc');
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" ".
+       '/etc/mail/sendmail.mc');
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}chmod -v 666 /etc/mail/sendmail.cf",'__display__');
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}m4 -d /etc/mail/sendmail.mc > /etc/mail/sendmail.cf");
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}chmod -v 644 /etc/mail/sendmail.cf",'__display__');
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}/etc/init.d/sendmail restart",'__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}git clone ".
+      'https://github.com/pztrn/statusnet-questioncaptcha-plugin',
+      '__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}cp -Rv ".
+      'statusnet-questioncaptcha-plugin/QuestionCaptcha '.
+      '/var/www/gnusocial/plugins','__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}rm -rfv ".
+      'statusnet-questioncaptcha-plugin','__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}php ".
+      '/var/www/gnusocial/plugins/QuestionCaptcha/scripts/'.
+      'generate-codes.php -c 10 -l 10','__display__');
+print "STDOUT=$stdout<== and ERROR=$stderr\n";
+   $stdout=~s/^.*(addPlugin.*)$/$1/s;
+   my $config_email=<<END;
+\\x24config['mail']['domain'] = 'fullauto.com';
+\\x24config['mail']['notifyfrom'] = \'$verified_email\';
+\\x24config['mail']['backend'] = 'sendmail';
+\\x24config['mail']['params'] = array(
+'host' => \'$smtp_server\',
+'port' => 25,
+'auth' => true,
+'username' => \'$access_id\',
+'password' => \'$smtppass\'
+);
+addPlugin('EmailRegistration');
+$stdout
 END
    ($stdout,$stderr)=$handle->cmd(
-      "echo -e \"$authinfo\" > /etc/mail/authinfo");
+      "${sudo}echo -e \"$config_email\" >> /var/www/gnusocial/config.php");
+   ($stdout,$stderr)=$handle->cmd(
+      "php /var/www/gnusocial/scripts/checkschema.php");
+   print "\n   ACCESS GNU SOCIAL UI AT:\n\n",
+         " https://$public_ip/index.php\n";
    my $thanks=<<'END';
 
      ______                  _    ,
@@ -1179,7 +1274,7 @@ END
    will be sending an email to this address that you must respond to
    in order for this address to work with your GNU Social installation.
    You can take a few minutes to do this now, or after you complete this
-   installation. Just note, that email in you GNU Social server will
+   installation. Just note, that email in your GNU Social server will
    not work until you have responded to Amazon's email.
 
 END
